@@ -1,33 +1,42 @@
       subroutine pam(nn,jpp,kk,x,dys,jdyss,valmd,jtmd,ndyst,nsend,
      f     nrepr,nelem,radus,damer,ttd,separ,ttsyl,med,obj,ncluv,
      f     clusinf,sylinf,nisol)
-c     
+c
 c     partitioning around medoids
-c     
+c
 c     carries out a clustering using the k-medoid approach.
-c     
-      implicit double precision (a-h,o-z)
-      dimension x(nn,jpp),valmd(jpp),jtmd(jpp)
-      dimension nsend(nn),nrepr(nn),nelem(nn),ncluv(nn)
-      dimension radus(nn),damer(nn),ttd(nn),separ(nn),med(kk)
-      dimension clusinf(kk,5),sylinf(nn,4),obj(2)
-      dimension dys(1+nn*(nn-1)/2),nisol(kk)
+c
+      integer nn,jpp,kk, jdyss,ndyst
 
-      if(jdyss.eq.1)go to 125
-      jhalt=0
-      call dysta(nn,jpp,x,dys,ndyst,jtmd,valmd,jhalt)
-      if (jhalt.eq.0)go to 125
-      jdyss=-1
-      return
-c     
- 125  s=0.0
+      double precision x(nn,jpp), dys(1+nn*(nn-1)/2), valmd(jpp)
+      integer jtmd(jpp), nsend(nn),nrepr(nn),nelem(nn),
+     +     ncluv(nn), nisol(kk), med(kk)
+      double precision radus(nn),damer(nn),ttd(nn),separ(nn),ttsyl
+      double precision clusinf(kk,5),sylinf(nn,4),obj(2)
+c
+      integer jhalt, nhalf, k,l
+      double precision sky,s
+
+c jdyss = 0 : compute distances from x
+c       = 1 : distances provided  in x
+      if(jdyss.ne.1) then
+         jhalt=0
+         call dysta(nn,jpp,x,dys,ndyst,jtmd,valmd,jhalt)
+         if (jhalt.ne.0) then
+            jdyss=-1
+            return
+         endif
+      endif
+c
+      s=0.0
       nhalf=nn*(nn-1)/2+1
       l=1
  130  l=l+1
       if(dys(l).gt.s)s=dys(l)
       if(l.lt.nhalf)go to 130
-      call bswap(kk,nn,nrepr,radus,damer,ttd,nhalf,dys,sky,s,obj)
-      call cstat(kk,nn,nsend,nrepr,radus,damer,ttd,separ,sky,s,
+
+      call bswap(kk,nn,      nrepr,radus,damer,ttd, nhalf,dys,sky,s,obj)
+      call cstat(kk,nn,nsend,nrepr,radus,damer,ttd, separ,    sky,s,
      f     nhalf,dys,ncluv,nelem,med,nisol)
       do 135 k=1,kk
          clusinf(k,1)=nrepr(k)
@@ -36,61 +45,70 @@ c
          clusinf(k,4)=damer(k)
          clusinf(k,5)=separ(k)
  135  continue
-      if(kk.le.1)go to 140
-      if(kk.ge.nn)go to 140
-      call dark(kk,nn,nhalf,ncluv,nsend,nelem,nrepr,
-     f     radus,damer,ttd,ttsyl,dys,s,sylinf)
- 140  end
+      if(1 .lt. kk .and. kk .lt. nn) then
+         call dark(kk,nn,nhalf,ncluv,nsend,nelem,nrepr,
+     f        radus,damer,ttd,ttsyl,dys,s,sylinf)
+      endif
+      end
 c     -----------------------------------------------------------
-c     
+
+c     Compute Distances from X matrix
+c
       subroutine dysta(nn,jpp,x,dys,ndyst,jtmd,valmd,jhalt)
 
-      implicit double precision (a-h,o-z)
-      dimension x(nn,jpp),dys(1+nn*(nn-1)/2),jtmd(jpp),valmd(jpp)
+      integer nn, jpp, ndyst, jtmd(jpp), jhalt
+      double precision x(nn,jpp), dys(1+nn*(nn-1)/2), valmd(jpp)
+
+      integer nlk,j,l,k, lsubt, npres
+      double precision pp, clk, rpres
 
       pp=jpp
-      nlk=1 
+      nlk=1
       dys(1)=0.0
-      do 100 l=2,nn 
-         lsubt=l-1 
-         do 20 k=1,lsubt 
-            clk=0.0 
-            nlk=nlk+1 
-            npres=0 
-            do 30 j=1,jpp 
-               if(jtmd(j).ge.0)goto 40 
-               if(x(l,j).eq.valmd(j))goto 30 
-               if(x(k,j).eq.valmd(j))goto 30 
- 40            npres=npres+1 
-               if(ndyst.ne.1)goto 50 
-               clk=clk+(x(l,j)-x(k,j))*(x(l,j)-x(k,j)) 
-               goto 30 
+      do 100 l=2,nn
+         lsubt=l-1
+         do 20 k=1,lsubt
+            clk=0.0
+            nlk=nlk+1
+            npres=0
+            do 30 j=1,jpp
+               if(jtmd(j).ge.0)goto 40
+               if(x(l,j).eq.valmd(j))goto 30
+               if(x(k,j).eq.valmd(j))goto 30
+ 40            npres=npres+1
+               if(ndyst.ne.1)goto 50
+               clk=clk+(x(l,j)-x(k,j))*(x(l,j)-x(k,j))
+               goto 30
  50            clk=clk+dabs(x(l,j)-x(k,j))
  30         continue
-            rpres=npres 
-            if(npres.ne.0)goto 60 
-            jhalt=1 
+            rpres=npres
+            if(npres.ne.0)goto 60
+            jhalt=1
             dys(nlk)=-1.0
-            goto 20 
- 60         if(ndyst.ne.1)goto 70 
-            dys(nlk)=dsqrt(clk*(pp/rpres)) 
-            goto 20 
- 70         dys(nlk)=clk*(pp/rpres) 
+            goto 20
+ 60         if(ndyst.ne.1)goto 70
+            dys(nlk)=dsqrt(clk*(pp/rpres))
+            goto 20
+ 70         dys(nlk)=clk*(pp/rpres)
  20      continue
  100  continue
-      end 
+      end
 c     -----------------------------------------------------------
-c     
+c
       subroutine bswap(kk,nn,nrepr,dysma,dysmb,beter,hh,dys,sky,s,obj)
 
-      implicit double precision (a-h,o-z)
-
       integer kk,nn,nrepr(nn),hh
       double precision dysma(nn),dysmb(nn),beter(nn),dys(hh), sky,s,
      +     obj(2)
-c     
+c function called
+      integer meet
+      external meet
+c
+      integer nny, j,ja,k,nkj, njn,njaj,nmax, nbest,kbest
+      double precision cmd, ammax, rnn,dz,dzsky,small
+c
 c     first algorithm: build.
-c     
+c
       nny=0
       do 17 j=1,nn
          nrepr(j)=0
@@ -129,9 +147,9 @@ C-- Loop :
       rnn=nn
       obj(1)=sky/rnn
       if(kk.eq.1)goto 75
-c     
+c
 c     second algorithm: swap.
-c     
+c
 C-- Loop :
  60   do 63 j=1,nn
          dysma(j)=1.1*s+1.0
@@ -179,15 +197,21 @@ C-- Loop :
       obj(2)=sky/rnn
       end
 c     -----------------------------------------------------------
-c     
+c
       subroutine cstat(kk,nn,nsend,nrepr,radus,damer,ttd,separ,z,s,
      f     hh,dys,ncluv,nelem,med,nisol)
 
-      implicit double precision (a-h,o-z)
-      integer hh
-      dimension ncluv(nn),nsend(nn),nrepr(nn),nelem(nn)
-      dimension separ(nn),damer(nn),radus(nn),ttd(nn)
-      dimension dys(hh),med(kk),nisol(kk)
+      integer kk,nn, hh, nsend(nn),nrepr(nn)
+      integer ncluv(nn),nelem(nn), nisol(kk), med(kk)
+      double precision radus(nn),damer(nn),ttd(nn),separ(nn),z,s,dys(hh)
+
+c function called
+      integer meet
+      external meet
+c
+      integer j,k,m,ja,jb,jk, njaj, ksmal,nplac,numcl,numl,nel,njm,
+     +     nvn,ntt,mevj,kand,nvna,jndz
+      double precision dsmal, ttt,rtt,rnn,dam,sep,aja,ajb
 
       do 130 j=1,nn
          if(nrepr(j).eq.1)go to 120
@@ -218,9 +242,9 @@ c
  140     continue
          if(jk.eq.kk)go to 148
  145  continue
-c     
+c
 c     analysis of the clustering.
-c     
+c
  148  do 160 numcl=1,kk
          ntt=0
          radus(numcl)=-1.0
@@ -243,16 +267,16 @@ c
       damer(1)=s
       nrepr(1)=nn
       go to 300
-c     
+c
 c     numl = number of l-clusters.
-c     
+c
  240  numl=0
       do 40 k=1,kk
-c     
+c
 c     identification of cluster k:
-c     nel=number of objects
-c     nelem=vector of objects
-c     
+c     nel  = number of objects
+c     nelem= vector of objects
+c
          nel=0
          do 23 j=1,nn
             if(ncluv(j).ne.k)go to 23
@@ -267,12 +291,13 @@ c
          do 250 j=1,nn
             if(j.eq.nvn)go to 250
             mevj=meet(nvn,j)
-            if(separ(k).gt.dys(mevj))separ(k)=dys(mevj)
+            if(separ(k).gt.dys(mevj)) separ(k)=dys(mevj)
  250     continue
-c     
-c     is cluster k     1) an l-cluster ?
-c     2) an l*-cluster ?
-c     
+c
+c Is cluster k
+c	1) an l-cluster  or
+c       2) an l*-cluster ?
+c
          if(separ(k).eq.0.)go to 400
          numl=numl+1
  400     go to 35
@@ -285,14 +310,15 @@ c
             ajb=1.1*s+1.0
             do 25 jb=1,nn
                jndz=meet(nvna,jb)
-               if(ncluv(jb).eq.k)go to 30
-               if(dys(jndz).lt.ajb)ajb=dys(jndz)
-               go to 25
- 30            if(dys(jndz).gt.aja)aja=dys(jndz)
+               if(ncluv(jb).eq.k) then
+                  if(dys(jndz).gt.aja) aja=dys(jndz)
+               else
+                  if(dys(jndz).lt.ajb) ajb=dys(jndz)
+               endif
  25         continue
-            if(aja.ge.ajb)kand=0
-            if(dam.lt.aja)dam=aja
-            if(sep.gt.ajb)sep=ajb
+            if(aja.ge.ajb) kand=0
+            if(dam.lt.aja) dam=aja
+            if(sep.gt.ajb) sep=ajb
  26      continue
          separ(k)=sep
          damer(k)=dam
@@ -303,89 +329,96 @@ c     l-cluster
          nisol(k)=1
          go to 40
 c     l*-cluster
- 27      nisol(k)=2 
+ 27      nisol(k)=2
          go to 40
  35      nisol(k)=0
  40   continue
  300  end
 c     -----------------------------------------------------------
-c     
+c
       subroutine dark(kk,nn,hh,ncluv,nsend,nelem,negbr,
      f     syl,srank,avsyl,ttsyl,dys,s,sylinf)
 
-      implicit double precision (a-h,o-z)
-      integer hh
-      dimension ncluv(nn),nsend(nn),nelem(nn),negbr(nn)
-      dimension syl(nn),srank(nn),avsyl(nn),dys(hh),sylinf(nn,4)
+      integer kk,nn,hh
+      integer ncluv(nn),nsend(nn),nelem(nn),negbr(nn)
+      double precision syl(nn),srank(nn),avsyl(nn),dys(hh),sylinf(nn,4)
+      double precision ttsyl, s
+c function called
+      integer meet
+      external meet
+c
+      integer j,l, lang,lplac, nsylr, nclu,numcl,ntt,nj,njl,nl,nbb,mjl
+      double precision dysa,dysb,db,att,btt,rtt,rnn,symax
+
       nsylr=0
-      ttsyl=0.0 
-      do 100 numcl=1,kk 
-         ntt=0 
+      ttsyl=0.0
+      do 100 numcl=1,kk
+         ntt=0
          do 30 j=1,nn
             if(ncluv(j).ne.numcl)go to 30
             ntt=ntt+1
             nelem(ntt)=j
  30      continue
-         do 40 j=1,ntt 
-            nj=nelem(j) 
+         do 40 j=1,ntt
+            nj=nelem(j)
             dysb=1.1*s+1.0
             negbr(j)=-1
             do 41 nclu=1,kk
                if(nclu.eq.numcl)go to 41
-               nbb=0 
+               nbb=0
                db=0.0
                do 43 l=1,nn
                   if(ncluv(l).ne.nclu)go to 43
-                  nbb=nbb+1 
+                  nbb=nbb+1
                   mjl=meet(nj,l)
                   db=db+dys(mjl)
  43            continue
-               btt=nbb 
-               db=db/btt 
+               btt=nbb
+               db=db/btt
                if(db.ge.dysb)go to 41
-               dysb=db 
-               negbr(j)=nclu 
+               dysb=db
+               negbr(j)=nclu
  41         continue
             if(ntt.eq.1)go to 50
             dysa=0.0
-            do 45 l=1,ntt 
-               nl=nelem(l) 
-               njl=meet(nj,nl) 
+            do 45 l=1,ntt
+               nl=nelem(l)
+               njl=meet(nj,nl)
                dysa=dysa+dys(njl)
  45         continue
             att=ntt-1
             dysa=dysa/att
-            if(dysa.gt.0.0)go to 51 
+            if(dysa.gt.0.0)go to 51
             if(dysb.gt.0.0)go to 52
  50         syl(j)=0.0
             go to 40
  52         syl(j)=1.0
             go to 40
- 51         if(dysb.le.0.0)go to 53 
+ 51         if(dysb.le.0.0)go to 53
             if(dysb.gt.dysa)syl(j)=1.0-dysa/dysb
             if(dysb.lt.dysa)syl(j)=dysb/dysa-1.0
             if(dysb.eq.dysa)syl(j)=0.0
             go to 54
- 53         syl(j)=-1.0 
- 54         if(syl(j).le.(-1.0))syl(j)=-1.0 
-            if(syl(j).ge.1.0)syl(j)=1.0 
+ 53         syl(j)=-1.0
+ 54         if(syl(j).le.(-1.0))syl(j)=-1.0
+            if(syl(j).ge.1.0)syl(j)=1.0
  40      continue
          avsyl(numcl)=0.0
-         do 60 j=1,ntt 
+         do 60 j=1,ntt
             symax=-2.0
             do 70 l=1,ntt
-               if(syl(l).le.symax)go to 70 
+               if(syl(l).le.symax)go to 70
                symax=syl(l)
                lang=l
  70         continue
-            nsend(j)=lang 
+            nsend(j)=lang
             srank(j)=syl(lang)
             avsyl(numcl)=avsyl(numcl)+srank(j)
             syl(lang)=-3.0
  60      continue
          ttsyl=ttsyl+avsyl(numcl)
-         rtt=ntt 
-         avsyl(numcl)=avsyl(numcl)/rtt 
+         rtt=ntt
+         avsyl(numcl)=avsyl(numcl)/rtt
 
          if(ntt.ge.2)goto 75
          nsylr=nsylr+1
@@ -394,7 +427,7 @@ c
          sylinf(nsylr,3)=0.0
          sylinf(nsylr,4)=nelem(1)
          goto 100
- 75      do 80 l=1,ntt 
+ 75      do 80 l=1,ntt
             nsylr=nsylr+1
             lplac=nsend(l)
             sylinf(nsylr,1)=numcl
@@ -404,5 +437,5 @@ c
  80      continue
  100  continue
       rnn=nn
-      ttsyl=ttsyl/rnn 
+      ttsyl=ttsyl/rnn
  96   end
